/*
    i2cset.c - A user-space program to write an I2C register.
    Copyright (c) 2001  Frodo Looijaard <frodol@dds.nl>, and
    Mark D. Studebaker <mdsxyz123@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    17-Apr-2004 Modified version for I2C project by NekMech.
*/

#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

void help(void)
{
  FILE *fptr;
  char s[100];

  printf("Syntax: i2cread I2CBUS CHIP-ADDRESS COUNT\n");
  printf("  I2CBUS is an integer\n");
  if((fptr = fopen("/proc/bus/i2c", "r"))) {
    printf("  Installed I2C busses:\n");
    while(fgets(s, 100, fptr))
      printf("    %s", s);	
    fclose(fptr);
  }
  exit(1);
}

int main(int argc, char *argv[])
{
  char *end;
  int i2cbus,address,count,file;
  int loop;
  int e1, e2, e3;
  char filename1[20];
  char filename2[20];
  char filename3[20];
  char *filename;
  char buf[10];

  if (argc < 3)
    help();


  i2cbus = strtol(argv[1],&end,0);
  if (*end || (i2cbus < 0) || (i2cbus > 0x3f)) {
    printf("Error: I2CBUS argument invalid!\n");
    help();
  }

  address = strtol(argv[2],&end,0);
  if (*end || (address < 0) || (address > 0x7f)) {
    printf("Error: Chip address invalid!\n");
    help();
  }

  count = 1;
  if (argc > 3) {
     count = strtol(argv[3],&end,0);
     if (*end || (count < 1) || (count > 10)) {
       printf("Error: count is invalid! Must be in range 1 to 10\n");
       help();
     }
  }

/*
 * Try all three variants and give the correct error message
 * upon failure
 */

  sprintf(filename1,"/dev/i2c-%d",i2cbus);
  sprintf(filename2,"/dev/i2c%d",i2cbus);
  sprintf(filename3,"/dev/i2c/%d",i2cbus);
  if ((file = open(filename1,O_RDWR)) < 0) {
    e1 = errno;
    if ((file = open(filename2,O_RDWR)) < 0) {
      e2 = errno;
      if ((file = open(filename3,O_RDWR)) < 0) {
        e3 = errno;
        if(e1 == ENOENT && e2 == ENOENT && e3 == ENOENT) {
          printf("Error: Could not open file `%s', `%s', or `%s': %s\n",
                     filename1,filename2,filename3,strerror(ENOENT));
        }
        if (e1 != ENOENT) {
          printf("Error: Could not open file `%s' : %s\n",
                     filename1,strerror(e1));
          if(e1 == EACCES)
            printf("Run as root?\n");
        }
        if (e2 != ENOENT) {
          printf("Error: Could not open file `%s' : %s\n",
                     filename2,strerror(e2));
          if(e2 == EACCES)
            printf("Run as root?\n");
        }
        if (e3 != ENOENT) {
          printf("Error: Could not open file `%s' : %s\n",
                     filename3,strerror(e3));
          if(e3 == EACCES)
            printf("Run as root?\n");
        }
        exit(1);
      } else {
         filename = filename3;
      }
    } else {
       filename = filename2;
    }
  } else {
    filename = filename1;
  }
  
  if (ioctl(file,I2C_SLAVE,address) < 0) {
    /* ERROR HANDLING; you can check errno to see what went wrong */
    printf("Error: Could not set address to %d: %s\n",address,
            strerror(errno));
    exit(1);
  }

  /* Using I2C Read bytes */
  if (read(file,buf,count) != count) {
    /* ERROR HANDLING: i2c transaction failed */
    printf ("Failed to read bytes\n");
    exit(1);
  } else {
    /* buf array contains the read byte */
    printf ("Returned bytes:");
    for (loop = 0; loop < count; ++loop)
       printf (" %i",buf[loop]);
    printf ("\n");
  }
  close(file);
  exit(0);
}
